import {
  Component,
  DoCheck,
  ElementRef,
  EventEmitter,
  Input,
  OnChanges,
  OnDestroy,
  OnInit,
  Output,
  Query,
  ViewChild,
  ViewContainerRef,
} from '@angular/core';
import { AbstractControlDirective, NgControl } from '@angular/forms';
import { Observable } from 'rxjs/Observable';

@Component({
  selector: 'rl-datepicker-input',
  styles: [`
    :host {
      position : relative;
    }
  `],
  template: `
    <div class="input-group">
      <input class="input-sm form-control" name="datepicker"
            [value]="dateObject | date: options.format" type="text" [disabled]="inputDisabled" (click)="opened = true"/>
      <div class="input-group-btn">
        <button class="btn btn-default" [disabled]="buttonDisabled" type="button" (click)="open()" #OpenBtn>
          <span class="glyphicon glyphicon-calendar"></span>
        </button>
      </div>
    </div>
    <div rlDatepickerPopup>
      <datepicker *ngIf="! options.overrideInternalDatepicker"
                  [(ngModel)]="dateObject" name="datepicker" [hidden]="!opened" [minDate]="options.minDate"
                  [maxDate]="options.maxDate" [showWeeks]="options.showWeeks"
                  (selectionDone)="selectionDone(dateObject)"></datepicker>
      
      <ng-content></ng-content>
      <div [hidden]="!opened" style="padding: 5px;">
        <span class="btn-group pull-left">       
          <button type="button" class="btn btn-sm btn-info"
                  (click)="setToToday()">Today</button>
          <button type="button" class="btn btn-sm btn-danger"
                  (click)="clearDate()">Clear</button>
        </span>
        <button type="button" class="btn btn-sm btn-success pull-right" (click)="opened = false">Done</button>
      </div>
    </div>
  `
})

export class RlDatePickerInputComponent implements OnInit, DoCheck, OnDestroy, OnChanges {

  private defaultOptions = {
    overrideInternalDatepicker: false,
    format: 'MM/dd/yyyy',
    minDate: null,
    maxDate: null,
    showWeeks: false
  };

  @ViewChild('OpenBtn') public OpenBtn: ElementRef;
  @Input('ngModel') ngModel: AbstractControlDirective;
  @Input() inputDisabled = false;
  @Input() buttonDisabled = false;

  @Input() options: any;

  @Output() onClose: EventEmitter<any> = new EventEmitter();

  public opened: boolean = false;
  public dateObject = new Date();

  //keep a copy to compare
  protected $dateObject;
  protected docmousedown;
  protected subscriber;

  constructor(protected el: ElementRef,
    protected control: NgControl) {
  }

  ngOnInit(): void {
    this.dateObject = this.ngModel || this.control.value;

    this.options = Object.assign(this.defaultOptions, this.options);
  }

  ngOnDestroy = (): void => this.removeEventListener();

  ngOnChanges(changes) {
    if (changes.options) {
      this.options = Object.assign(this.defaultOptions, this.options);
    }
  }

  ngDoCheck(): void {
    // if (this.dateObject !== this.$dateObject) {
    //   this.$dateObject = this.dateObject;
    //   this.control.control.setValue(this.dateObject);
    // }

    if (!this.opened && this.subscriber && !this.subscriber.closed) {
      setTimeout(() => {
        this.removeEventListener();
        this.onClose.emit();
      }, 0)
    } else if (this.opened && (!this.subscriber || (this.subscriber && this.subscriber.closed))) {
      setTimeout(() => {
        this.addEventListener();
      }, 0);
    }
  }

  /**
   * Get the HTMLElement of this Element
   * @returns {HTMLElement}
   * @constructor
   */
  public get LayoutElement(): HTMLElement {
    return <HTMLElement>(<Element>this.el.nativeElement);
  }

  /**
   * Get the HTMLElement of the Datepicker
   * @returns {HTMLElement}
   * @constructor
   */
  public get DatepickerElement(): HTMLElement {
    return <HTMLElement>(<Element>this.LayoutElement.children[1]);
  }

  public get OpenBtnElement() {
    return <HTMLElement>(<Element>this.OpenBtn.nativeElement || this.OpenBtn);
  }

  public removeEventListener() {
    if (this.subscriber)
      this.subscriber.unsubscribe();
  }

  public selectionDone(dateObject: Date){
    setTimeout(() => {
      this.opened = false;
      this.control.control.setValue(this.dateObject);
    }, 100);
  }

  public addEventListener() {
    this.docmousedown = Observable.fromEvent(document, 'mousedown');

    this.subscriber = this.docmousedown.subscribe((e) => {
      e.stopPropagation();
      e.preventDefault();

      const btnClicked = this.OpenBtnElement.contains(e.target) === false;
      const dtClicked = this.DatepickerElement.contains(e.target) === false;

      if (btnClicked && dtClicked)
        this.opened = false;

    });
  }

  open() {
    this.opened = !this.opened;
    this.opened ? this.addEventListener() : this.removeEventListener();
  }

  setToToday() {
    this.dateObject = new Date();
    this.opened = false;
    this.control.control.setValue(this.dateObject);
  }

  clearDate() {
    this.dateObject = undefined;
    this.opened = false;
    this.control.control.setValue(this.dateObject);
  }
}


